package org.jenkinsci.plugins.p4_client.review;

import hudson.Extension;
import hudson.Launcher;
import hudson.model.BuildListener;
import hudson.model.Environment;
import hudson.model.Result;
import hudson.model.AbstractBuild;
import hudson.model.Run;
import hudson.model.Run.RunnerAbortedException;
import hudson.model.listeners.RunListener;

import java.io.IOException;
import java.util.Map;
import java.util.logging.Logger;

import org.apache.http.HttpResponse;
import org.apache.http.client.HttpClient;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.impl.client.HttpClientBuilder;

@Extension
@SuppressWarnings("rawtypes")
public class ReviewNotifier extends RunListener<Run> {

	private static Logger logger = Logger.getLogger(ReviewNotifier.class.getName());

	private String fail;
	private String pass;

	public ReviewNotifier() {
		super(Run.class);
	}

	@Override
	public Environment setUpEnvironment(AbstractBuild build, Launcher launcher,
			BuildListener listener) throws IOException, InterruptedException,
			RunnerAbortedException {

		// Ensure Urls are reset from previous build
		fail = null;
		pass = null;

		@SuppressWarnings("unchecked")
		Map<String, String> map = build.getBuildVariables();

		if (map != null) {
			if (map.containsKey("fail")) {
				fail = map.get("fail");
			}
			if (map.containsKey("pass")) {
				pass = map.get("pass");
			}
		}

		return new Environment() {
		};
	}

	@Override
	public void onFinalized(Run r) {
		String url = fail;

		Result result = r.getResult();
		if (result.equals(Result.SUCCESS)) {
			url = pass;
		}

		try {
			if (url != null && !url.isEmpty()) {
				postURL(url);
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
		return;
	}

	private void postURL(String url) throws Exception {
		HttpClient client = HttpClientBuilder.create().build();
		HttpPost post = new HttpPost(url);

		HttpResponse response = client.execute(post);
		logger.info("Response code: " + response);
	}
}
